///<reference path="./../../../.d.ts"/>
var util = require("util");
var Future = require("fibers/future");
var helpers = require("./../../helpers");
var assert = require("assert");
var constants = require("../constants");
var DevicesServices = (function () {
    function DevicesServices($logger, $errors, $iOSDeviceDiscovery, $androidDeviceDiscovery, $staticConfig, $mobileHelper) {
        this.$logger = $logger;
        this.$errors = $errors;
        this.$iOSDeviceDiscovery = $iOSDeviceDiscovery;
        this.$androidDeviceDiscovery = $androidDeviceDiscovery;
        this.$staticConfig = $staticConfig;
        this.$mobileHelper = $mobileHelper;
        this.devices = {};
        this.platforms = [];
        this._isInitialized = false;
        this.attachToDeviceDiscoveryEvents();
    }
    Object.defineProperty(DevicesServices.prototype, "platform", {
        get: function () {
            return this._platform;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DevicesServices.prototype, "deviceCount", {
        get: function () {
            return this._device ? 1 : this.getDevices().length;
        },
        enumerable: true,
        configurable: true
    });
    DevicesServices.prototype.getDevices = function () {
        return _.values(this.devices);
    };
    DevicesServices.prototype.getAllPlatforms = function () {
        var _this = this;
        if (this.platforms.length > 0) {
            return this.platforms;
        }
        this.platforms = _.filter(this.$mobileHelper.platformNames, function (platform) { return _this.$mobileHelper.getPlatformCapabilities(platform).cableDeploy; });
        return this.platforms;
    };
    DevicesServices.prototype.getPlatform = function (platform) {
        var allSupportedPlatforms = this.getAllPlatforms();
        var normalizedPlatform = this.$mobileHelper.validatePlatformName(platform);
        if (!_.contains(allSupportedPlatforms, normalizedPlatform)) {
            this.$errors.failWithoutHelp("Deploying to %s connected devices is not supported. Build the " +
                "app using the `build` command and deploy the package manually.", normalizedPlatform);
        }
        return normalizedPlatform;
    };
    DevicesServices.prototype.attachToDeviceDiscoveryEvents = function () {
        this.$iOSDeviceDiscovery.deviceFound.add(this.onDeviceFound, this);
        this.$iOSDeviceDiscovery.deviceLost.add(this.onDeviceLost, this);
        this.$androidDeviceDiscovery.deviceFound.add(this.onDeviceFound, this);
        this.$androidDeviceDiscovery.deviceLost.add(this.onDeviceLost, this);
    };
    DevicesServices.prototype.onDeviceFound = function (device) {
        this.$logger.trace("Found device with identifier '%s'", device.deviceInfo.identifier);
        this.devices[device.deviceInfo.identifier] = device;
    };
    DevicesServices.prototype.onDeviceLost = function (device) {
        this.$logger.trace("Lost device with identifier '%s'", device.deviceInfo.identifier);
        delete this.devices[device.deviceInfo.identifier];
    };
    DevicesServices.prototype.startLookingForDevices = function () {
        var _this = this;
        return (function () {
            _this.$logger.trace("startLookingForDevices; platform is %s", _this._platform);
            if (!_this._platform) {
                _this.$iOSDeviceDiscovery.startLookingForDevices().wait();
                _this.$androidDeviceDiscovery.startLookingForDevices().wait();
            }
            else if (_this.$mobileHelper.isiOSPlatform(_this._platform)) {
                _this.$iOSDeviceDiscovery.startLookingForDevices().wait();
            }
            else if (_this.$mobileHelper.isAndroidPlatform(_this._platform)) {
                _this.$androidDeviceDiscovery.startLookingForDevices().wait();
            }
        }).future()();
    };
    DevicesServices.prototype.getAllConnectedDevices = function () {
        if (!this._platform) {
            return this.getDevices();
        }
        else {
            return this.filterDevicesByPlatform();
        }
    };
    DevicesServices.prototype.getDeviceByIndex = function (index) {
        this.validateIndex(index - 1);
        return this.getDevices()[index - 1];
    };
    DevicesServices.prototype.getDeviceByIdentifier = function (identifier) {
        var searchedDevice = _.find(this.getDevices(), function (device) { return device.deviceInfo.identifier === identifier; });
        if (!searchedDevice) {
            this.$errors.fail(DevicesServices.NOT_FOUND_DEVICE_BY_IDENTIFIER_ERROR_MESSAGE, identifier, this.$staticConfig.CLIENT_NAME.toLowerCase());
        }
        return searchedDevice;
    };
    DevicesServices.prototype.getDevice = function (deviceOption) {
        var _this = this;
        return (function () {
            _this.startLookingForDevices().wait();
            var device = null;
            if (_this.hasDevice(deviceOption)) {
                device = _this.getDeviceByIdentifier(deviceOption);
            }
            else if (helpers.isNumber(deviceOption)) {
                device = _this.getDeviceByIndex(parseInt(deviceOption, 10));
            }
            if (!device) {
                _this.$errors.fail("Cannot resolve the specified connected device by the provided index or identifier. To list currently connected devices and verify that the specified index or identifier exists, run '%s device'.", _this.$staticConfig.CLIENT_NAME.toLowerCase());
            }
            return device;
        }).future()();
    };
    DevicesServices.prototype.executeOnDevice = function (action, canExecute) {
        var _this = this;
        return (function () {
            if (!canExecute || canExecute(_this._device)) {
                action(_this._device).wait();
            }
        }).future()();
    };
    DevicesServices.prototype.executeOnAllConnectedDevices = function (action, canExecute) {
        var _this = this;
        return (function () {
            var allConnectedDevices = _this.getAllConnectedDevices();
            var futures = _.map(allConnectedDevices, function (device) {
                if (!canExecute || canExecute(device)) {
                    var future = action(device);
                    Future.settle(future);
                    return future;
                }
                else {
                    return Future.fromResult();
                }
            });
            Future.wait(futures);
        }).future()();
    };
    DevicesServices.prototype.execute = function (action, canExecute, options) {
        var _this = this;
        return (function () {
            assert.ok(_this._isInitialized, "Devices services not initialized!");
            if (_this.hasDevices) {
                if (_this._device) {
                    _this.executeOnDevice(action, canExecute).wait();
                }
                else {
                    _this.executeOnAllConnectedDevices(action, canExecute).wait();
                }
            }
            else {
                var message = constants.ERROR_NO_DEVICES;
                if (options && options["allowNoDevices"]) {
                    _this.$logger.info(message);
                }
                else {
                    _this.$errors.fail({ formatStr: message, suppressCommandHelp: true });
                }
            }
        }).future()();
    };
    DevicesServices.prototype.initialize = function (data) {
        var _this = this;
        var platform = data.platform;
        var deviceOption = data.deviceId;
        if (this._isInitialized) {
            return Future.fromResult();
        }
        return (function () {
            if (platform && deviceOption) {
                _this._device = _this.getDevice(deviceOption).wait();
                _this._platform = _this._device.deviceInfo.platform;
                if (_this._platform !== _this.getPlatform(platform)) {
                    _this.$errors.fail("Cannot resolve the specified connected device. The provided platform does not match the provided index or identifier." +
                        "To list currently connected devices and verify that the specified pair of platform and index or identifier exists, run 'device'.");
                }
                _this.$logger.warn("Your application will be deployed only on the device specified by the provided index or identifier.");
            }
            else if (!platform && deviceOption) {
                _this._device = _this.getDevice(deviceOption).wait();
                _this._platform = _this._device.deviceInfo.platform;
            }
            else if (platform && !deviceOption) {
                _this._platform = _this.getPlatform(platform);
                _this.startLookingForDevices().wait();
            }
            else if (!platform && !deviceOption) {
                _this.startLookingForDevices().wait();
                if (!data.skipInferPlatform) {
                    var devices = _this.getDevices();
                    var platforms = _.uniq(_.map(devices, function (device) { return device.deviceInfo.platform; }));
                    if (platforms.length === 1) {
                        _this._platform = platforms[0];
                    }
                    else {
                        if (platforms.length === 0) {
                            _this.$errors.fail({ formatStr: constants.ERROR_NO_DEVICES, suppressCommandHelp: true });
                        }
                        else {
                            _this.$errors.fail("Multiple device platforms detected (%s). Specify platform or device on command line.", helpers.formatListOfNames(platforms, "and"));
                        }
                    }
                }
            }
            _this._isInitialized = true;
        }).future()();
    };
    Object.defineProperty(DevicesServices.prototype, "hasDevices", {
        get: function () {
            if (!this._platform) {
                return this.getDevices().length !== 0;
            }
            else {
                return this.filterDevicesByPlatform().length !== 0;
            }
        },
        enumerable: true,
        configurable: true
    });
    DevicesServices.prototype.hasDevice = function (identifier) {
        return _.some(this.getDevices(), function (device) { return device.deviceInfo.identifier === identifier; });
    };
    DevicesServices.prototype.filterDevicesByPlatform = function () {
        var _this = this;
        return _.filter(this.getDevices(), function (device) { return device.deviceInfo.platform === _this._platform; });
    };
    DevicesServices.prototype.validateIndex = function (index) {
        if (index < 0 || index > this.getDevices().length) {
            throw new Error(util.format(DevicesServices.NOT_FOUND_DEVICE_BY_INDEX_ERROR_MESSAGE, index, this.$staticConfig.CLIENT_NAME.toLowerCase()));
        }
    };
    DevicesServices.NOT_FOUND_DEVICE_BY_IDENTIFIER_ERROR_MESSAGE = "Could not find device by specified identifier '%s'. To list currently connected devices and verify that the specified identifier exists, run '%s device'.";
    DevicesServices.NOT_FOUND_DEVICE_BY_INDEX_ERROR_MESSAGE = "Could not find device by specified index %d. To list currently connected devices and verify that the specified index exists, run '%s device'.";
    return DevicesServices;
})();
exports.DevicesServices = DevicesServices;
$injector.register("devicesServices", DevicesServices);
