///<reference path="../../.d.ts"/>
"use strict";
var minimatch = require("minimatch");
var path = require("path");
var util = require("util");
var gaze = require("gaze");
var UsbLiveSyncServiceBase = (function () {
    function UsbLiveSyncServiceBase($devicesServices, $mobileHelper, $localToDevicePathDataFactory, $logger, $options, $deviceAppDataFactory, $fs, $dispatcher, $childProcess, $iOSEmulatorServices, $hostInfo) {
        this.$devicesServices = $devicesServices;
        this.$mobileHelper = $mobileHelper;
        this.$localToDevicePathDataFactory = $localToDevicePathDataFactory;
        this.$logger = $logger;
        this.$options = $options;
        this.$deviceAppDataFactory = $deviceAppDataFactory;
        this.$fs = $fs;
        this.$dispatcher = $dispatcher;
        this.$childProcess = $childProcess;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$hostInfo = $hostInfo;
        this._initialized = false;
        this.timer = null;
        this.syncQueue = [];
    }
    UsbLiveSyncServiceBase.prototype.initialize = function (platform) {
        var _this = this;
        return (function () {
            if (!(_this.$options.emulator && platform && platform.toLowerCase() === "ios")) {
                _this.$devicesServices.initialize({ platform: platform, deviceId: _this.$options.device }).wait();
                _this._initialized = true;
                return _this.$devicesServices.platform;
            }
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.sync = function (platform, appIdentifier, projectFilesPath, excludedProjectDirsAndFiles, watchGlob, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, notRunningiOSSimulatorAction, localProjectRootPath, beforeLiveSyncAction, beforeBatchLiveSyncAction, iOSSimulatorRelativeToProjectBasePathAction) {
        var _this = this;
        return (function () {
            var synciOSSimulator = _this.$hostInfo.isDarwin ? _this.$iOSEmulatorServices.isSimulatorRunning().wait() || (_this.$options.emulator && platform.toLowerCase() === "ios") : false;
            if (synciOSSimulator) {
                _this.$iOSEmulatorServices.sync(appIdentifier, projectFilesPath, notRunningiOSSimulatorAction).wait();
            }
            if (!_this._initialized && (!_this.$options.emulator || platform.toLowerCase() === "android")) {
                _this.initialize(platform).wait();
            }
            if (!_this.$options.emulator || platform.toLowerCase() === "android") {
                var projectFiles = _this.$fs.enumerateFilesInDirectorySync(projectFilesPath, function (filePath, stat) { return !_this.isFileExcluded(path.relative(projectFilesPath, filePath), excludedProjectDirsAndFiles, projectFilesPath); }, { enumerateDirectories: true });
                _this.syncCore(platform, projectFiles, appIdentifier, localProjectRootPath || projectFilesPath, restartAppOnDeviceAction, notInstalledAppOnDeviceAction).wait();
            }
            if (_this.$options.watch) {
                var __this = _this;
                gaze("**/*", { cwd: watchGlob }, function (err, watcher) {
                    this.on('all', function (event, filePath) {
                        if (event === "added" || event === "changed") {
                            if (!_.contains(excludedProjectDirsAndFiles, filePath)) {
                                if (synciOSSimulator) {
                                    __this.$dispatcher.dispatch(function () { return __this.$iOSEmulatorServices.syncFiles(appIdentifier, projectFilesPath, [filePath], notRunningiOSSimulatorAction, iOSSimulatorRelativeToProjectBasePathAction); });
                                }
                                if (!__this.$options.emulator || platform.toLowerCase() === "android") {
                                    __this.batchLiveSync(platform, filePath, appIdentifier, localProjectRootPath || projectFilesPath, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, beforeLiveSyncAction, beforeBatchLiveSyncAction);
                                }
                            }
                        }
                    });
                });
                _this.$dispatcher.run();
            }
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.syncCore = function (platform, projectFiles, appIdentifier, projectFilesPath, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, beforeLiveSyncAction) {
        var _this = this;
        return (function () {
            platform = platform ? _this.$mobileHelper.normalizePlatformName(platform) : _this.$devicesServices.platform;
            var deviceAppData = _this.$deviceAppDataFactory.create(appIdentifier, platform);
            var localToDevicePaths = _(projectFiles)
                .map(function (projectFile) { return _this.getProjectFileInfo(projectFile); })
                .filter(function (projectFileInfo) { return projectFileInfo.shouldIncludeFile; })
                .map(function (projectFileInfo) { return _this.$localToDevicePathDataFactory.create(projectFileInfo.fileName, projectFilesPath, projectFileInfo.onDeviceName, deviceAppData.deviceProjectRootPath); })
                .value();
            var action = function (device) {
                return (function () {
                    if (deviceAppData.isLiveSyncSupported(device).wait()) {
                        if (beforeLiveSyncAction) {
                            beforeLiveSyncAction(device, deviceAppData).wait();
                        }
                        var applications = device.applicationManager.getInstalledApplications().wait();
                        if (!_.contains(applications, deviceAppData.appIdentifier)) {
                            _this.$logger.warn("The application with id \"" + deviceAppData.appIdentifier + "\" is not installed on the device yet.");
                            notInstalledAppOnDeviceAction(device).wait();
                            return;
                        }
                        _this.$logger.info("Transferring project files...");
                        device.fileSystem.transferFiles(deviceAppData.appIdentifier, localToDevicePaths).wait();
                        _this.$logger.info("Successfully transferred all project files.");
                        _this.$logger.info("Applying changes...");
                        restartAppOnDeviceAction(device, deviceAppData, localToDevicePaths).wait();
                        _this.$logger.info("Successfully synced application " + deviceAppData.appIdentifier + " on device " + device.deviceInfo.identifier + ".");
                    }
                }).future()();
            };
            _this.$devicesServices.execute(action).wait();
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.batchLiveSync = function (platform, filePath, appIdentifier, projectFilesPath, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, beforeLiveSyncAction, beforeBatchLiveSyncAction) {
        var _this = this;
        if (!this.timer) {
            this.timer = setInterval(function () {
                var filesToSync = _this.syncQueue;
                if (filesToSync.length > 0) {
                    _this.syncQueue = [];
                    _this.$logger.trace("Syncing %s", filesToSync.join(", "));
                    _this.$dispatcher.dispatch(function () {
                        return (function () {
                            _this.syncCore(platform, filesToSync, appIdentifier, projectFilesPath, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, beforeLiveSyncAction).wait();
                        }).future()();
                    });
                }
            }, 500);
        }
        this.$dispatcher.dispatch(function () { return (function () { _this.syncQueue.push(beforeBatchLiveSyncAction(filePath).wait()); }).future()(); });
    };
    UsbLiveSyncServiceBase.prototype.isFileExcluded = function (path, exclusionList, projectDir) {
        return !!_.find(exclusionList, function (pattern) { return minimatch(path, pattern, { nocase: true }); });
    };
    UsbLiveSyncServiceBase.prototype.getProjectFileInfo = function (fileName) {
        var parsed = this.parseFile(fileName, this.$mobileHelper.platformNames, this.$devicesServices.platform);
        if (!parsed) {
            parsed = this.parseFile(fileName, ["debug", "release"], "debug");
        }
        return parsed || {
            fileName: fileName,
            onDeviceName: fileName,
            shouldIncludeFile: true
        };
    };
    UsbLiveSyncServiceBase.prototype.parseFile = function (fileName, validValues, value) {
        var regex = util.format("^(.+?)[.](%s)([.].+?)$", validValues.join("|"));
        var parsed = fileName.match(new RegExp(regex, "i"));
        if (parsed) {
            return {
                fileName: fileName,
                onDeviceName: parsed[1] + parsed[3],
                shouldIncludeFile: parsed[2].toLowerCase() === value.toLowerCase(),
                value: value
            };
        }
        return undefined;
    };
    return UsbLiveSyncServiceBase;
})();
exports.UsbLiveSyncServiceBase = UsbLiveSyncServiceBase;
