///<reference path="../.d.ts"/>
"use strict";
var path = require("path");
var shelljs = require("shelljs");
var semver = require("semver");
var Future = require("fibers/future");
var constants = require("./../constants");
var xmlmerge = require("xmlmerge-js");
var DOMParser = require('xmldom').DOMParser;
var PluginsService = (function () {
    function PluginsService($platformsData, $npm, $fs, $projectData, $projectDataService, $childProcess, $options, $logger, $errors, $projectFilesManager) {
        this.$platformsData = $platformsData;
        this.$npm = $npm;
        this.$fs = $fs;
        this.$projectData = $projectData;
        this.$projectDataService = $projectDataService;
        this.$childProcess = $childProcess;
        this.$options = $options;
        this.$logger = $logger;
        this.$errors = $errors;
        this.$projectFilesManager = $projectFilesManager;
    }
    PluginsService.prototype.add = function (plugin) {
        var _this = this;
        return (function () {
            var dependencies = _this.getAllInstalledModules().wait();
            var dependencyData = _this.$npm.cache(plugin, undefined, PluginsService.NPM_CONFIG).wait();
            if (dependencyData.nativescript) {
                var pluginName = _this.executeNpmCommand(PluginsService.INSTALL_COMMAND_NAME, plugin).wait();
                _this.prepare(dependencyData).wait();
                _this.$logger.out("Successfully installed plugin " + dependencyData.name + ".");
            }
            else {
                _this.$errors.failWithoutHelp(plugin + " is not a valid NativeScript plugin. Verify that the plugin package.json file contains a nativescript key and try again.");
            }
        }).future()();
    };
    PluginsService.prototype.remove = function (pluginName) {
        var _this = this;
        return (function () {
            var removePluginNativeCodeAction = function (modulesDestinationPath, platform, platformData) {
                var pluginData = _this.convertToPluginData(_this.getNodeModuleData(pluginName).wait());
                pluginData.isPlugin = true;
                return platformData.platformProjectService.removePluginNativeCode(pluginData);
            };
            _this.executeForAllInstalledPlatforms(removePluginNativeCodeAction).wait();
            _this.executeNpmCommand(PluginsService.UNINSTALL_COMMAND_NAME, pluginName).wait();
            var showMessage = true;
            var action = function (modulesDestinationPath, platform, platformData) {
                return (function () {
                    shelljs.rm("-rf", path.join(modulesDestinationPath, pluginName));
                    _this.$logger.out("Successfully removed plugin " + pluginName + " for " + platform + ".");
                    showMessage = false;
                }).future()();
            };
            _this.executeForAllInstalledPlatforms(action).wait();
            if (showMessage) {
                _this.$logger.out("Succsessfully removed plugin " + pluginName);
            }
        }).future()();
    };
    PluginsService.prototype.prepare = function (dependencyData) {
        var _this = this;
        return (function () {
            var pluginData = _this.convertToPluginData(dependencyData);
            var action = function (pluginDestinationPath, platform, platformData) {
                return (function () {
                    var installedFrameworkVersion = _this.getInstalledFrameworkVersion(platform).wait();
                    var pluginPlatformsData = pluginData.platformsData;
                    if (pluginPlatformsData) {
                        var pluginVersion = pluginPlatformsData[platform];
                        if (!pluginVersion) {
                            _this.$logger.warn(pluginData.name + " is not supported for " + platform + ".");
                            return;
                        }
                        if (semver.gt(pluginVersion, installedFrameworkVersion)) {
                            _this.$logger.warn(pluginData.name + " " + pluginVersion + " for " + platform + " is not compatible with the currently installed framework version " + installedFrameworkVersion + ".");
                            return;
                        }
                    }
                    _this.$fs.ensureDirectoryExists(pluginDestinationPath).wait();
                    shelljs.cp("-Rf", pluginData.fullPath, pluginDestinationPath);
                    var pluginPlatformsFolderPath = path.join(pluginDestinationPath, pluginData.name, "platforms", platform);
                    var pluginConfigurationFilePath = path.join(pluginPlatformsFolderPath, platformData.configurationFileName);
                    var configurationFilePath = platformData.configurationFilePath;
                    if (_this.$fs.exists(pluginConfigurationFilePath).wait()) {
                        var pluginConfigurationFileContent = _this.$fs.readText(pluginConfigurationFilePath).wait();
                        _this.validateXml(pluginConfigurationFileContent, pluginConfigurationFilePath);
                        var configurationFileContent = _this.$fs.readText(configurationFilePath).wait();
                        _this.validateXml(configurationFileContent, configurationFilePath);
                        var resultXml = _this.mergeXml(configurationFileContent, pluginConfigurationFileContent, platformData.mergeXmlConfig || []).wait();
                        _this.validateXml(resultXml);
                        _this.$fs.writeFile(configurationFilePath, resultXml).wait();
                    }
                    _this.$projectFilesManager.processPlatformSpecificFiles(pluginDestinationPath, platform).wait();
                    pluginData.pluginPlatformsFolderPath = function (platform) { return path.join(pluginData.fullPath, "platforms", platform); };
                    platformData.platformProjectService.preparePluginNativeCode(pluginData).wait();
                    shelljs.rm("-rf", path.join(pluginDestinationPath, pluginData.name, "platforms"));
                    _this.$logger.out("Successfully prepared plugin " + pluginData.name + " for " + platform + ".");
                }).future()();
            };
            _this.executeForAllInstalledPlatforms(action).wait();
        }).future()();
    };
    PluginsService.prototype.ensureAllDependenciesAreInstalled = function () {
        var command = "npm install ";
        if (this.$options.ignoreScripts) {
            command += "--ignore-scripts";
        }
        return this.$childProcess.exec(command, { cwd: this.$projectData.projectDir });
    };
    PluginsService.prototype.getAllInstalledPlugins = function () {
        var _this = this;
        return (function () {
            var nodeModules = _this.getAllInstalledModules().wait();
            return _.filter(nodeModules, function (nodeModuleData) { return nodeModuleData && nodeModuleData.isPlugin; });
        }).future()();
    };
    Object.defineProperty(PluginsService.prototype, "nodeModulesPath", {
        get: function () {
            return path.join(this.$projectData.projectDir, "node_modules");
        },
        enumerable: true,
        configurable: true
    });
    PluginsService.prototype.getPackageJsonFilePath = function () {
        return path.join(this.$projectData.projectDir, "package.json");
    };
    PluginsService.prototype.getPackageJsonFilePathForModule = function (moduleName) {
        return path.join(this.nodeModulesPath, moduleName, "package.json");
    };
    PluginsService.prototype.getDependencies = function () {
        var packageJsonFilePath = this.getPackageJsonFilePath();
        return _.keys(require(packageJsonFilePath).dependencies);
    };
    PluginsService.prototype.getNodeModuleData = function (moduleName) {
        var _this = this;
        return (function () {
            var packageJsonFilePath = _this.getPackageJsonFilePathForModule(moduleName);
            if (_this.$fs.exists(packageJsonFilePath).wait()) {
                var data = require(packageJsonFilePath);
                return {
                    name: data.name,
                    version: data.version,
                    fullPath: path.dirname(packageJsonFilePath),
                    isPlugin: data.nativescript !== undefined,
                    moduleInfo: data.nativescript
                };
            }
            return null;
        }).future()();
    };
    PluginsService.prototype.convertToPluginData = function (cacheData) {
        var pluginData = {};
        pluginData.name = cacheData.name;
        pluginData.version = cacheData.version;
        pluginData.fullPath = path.dirname(this.getPackageJsonFilePathForModule(cacheData.name));
        pluginData.isPlugin = !!cacheData.nativescript;
        pluginData.pluginPlatformsFolderPath = function (platform) { return path.join(pluginData.fullPath, "platforms", platform); };
        if (pluginData.isPlugin) {
            pluginData.platformsData = cacheData.nativescript.platforms;
        }
        return pluginData;
    };
    PluginsService.prototype.getAllInstalledModules = function () {
        var _this = this;
        return (function () {
            _this.ensureAllDependenciesAreInstalled().wait();
            _this.$fs.ensureDirectoryExists(_this.nodeModulesPath).wait();
            var nodeModules = _this.getDependencies();
            return _.map(nodeModules, function (nodeModuleName) { return _this.getNodeModuleData(nodeModuleName).wait(); });
        }).future()();
    };
    PluginsService.prototype.executeNpmCommand = function (npmCommandName, npmCommandArguments) {
        var _this = this;
        return (function () {
            var result = "";
            if (npmCommandName === PluginsService.INSTALL_COMMAND_NAME) {
                result = _this.$npm.install(npmCommandArguments, _this.$projectData.projectDir, PluginsService.NPM_CONFIG).wait();
            }
            else if (npmCommandName === PluginsService.UNINSTALL_COMMAND_NAME) {
                result = _this.$npm.uninstall(npmCommandArguments, PluginsService.NPM_CONFIG).wait();
            }
            return _this.parseNpmCommandResult(result);
        }).future()();
    };
    PluginsService.prototype.parseNpmCommandResult = function (npmCommandResult) {
        return npmCommandResult[0][0].split("@")[0];
    };
    PluginsService.prototype.executeForAllInstalledPlatforms = function (action) {
        var _this = this;
        return (function () {
            var availablePlatforms = _.keys(_this.$platformsData.availablePlatforms);
            _.each(availablePlatforms, function (platform) {
                var isPlatformInstalled = _this.$fs.exists(path.join(_this.$projectData.platformsDir, platform.toLowerCase())).wait();
                if (isPlatformInstalled) {
                    var platformData = _this.$platformsData.getPlatformData(platform.toLowerCase());
                    var pluginDestinationPath = path.join(platformData.appDestinationDirectoryPath, constants.APP_FOLDER_NAME, "tns_modules");
                    action(pluginDestinationPath, platform.toLowerCase(), platformData).wait();
                }
            });
        }).future()();
    };
    PluginsService.prototype.getInstalledFrameworkVersion = function (platform) {
        var _this = this;
        return (function () {
            var platformData = _this.$platformsData.getPlatformData(platform);
            _this.$projectDataService.initialize(_this.$projectData.projectDir);
            var frameworkData = _this.$projectDataService.getValue(platformData.frameworkPackageName).wait();
            return frameworkData.version;
        }).future()();
    };
    PluginsService.prototype.mergeXml = function (xml1, xml2, config) {
        var future = new Future();
        try {
            xmlmerge.merge(xml1, xml2, config, function (mergedXml) {
                future.return(mergedXml);
            });
        }
        catch (err) {
            future.throw(err);
        }
        return future;
    };
    PluginsService.prototype.validateXml = function (xml, xmlFilePath) {
        var _this = this;
        var doc = new DOMParser({
            locator: {},
            errorHandler: function (level, msg) {
                var errorMessage = xmlFilePath ? "Invalid xml file " + xmlFilePath + "." : "Invalid xml " + xml + ".";
                _this.$errors.fail(errorMessage + (" Additional technical information: " + msg + "."));
            }
        });
        doc.parseFromString(xml, 'text/xml');
    };
    PluginsService.INSTALL_COMMAND_NAME = "install";
    PluginsService.UNINSTALL_COMMAND_NAME = "uninstall";
    PluginsService.NPM_CONFIG = {
        save: true
    };
    return PluginsService;
})();
exports.PluginsService = PluginsService;
$injector.register("pluginsService", PluginsService);
